# $Id: amazon.rb,v 1.6 2008/03/24 17:52:29 ianmacd Exp $
#
#:include: ../README.rdoc

module Amazon

  NAME = 'Ruby/Amazon'
  @@config = {}

  # Prints debugging messages and works like printf, except that it prints
  # only when Ruby is run with the -d switch.
  #
  def Amazon.dprintf(format='', *args)
    $stderr.printf( format + "\n", *args ) if $DEBUG
  end


  # Encode a string, such that it is suitable for HTTP transmission.
  #
  def Amazon.url_encode(string)
    # Shamelessly plagiarised from Wakou Aoyama's cgi.rb.
    #
    string.gsub( /([^ a-zA-Z0-9_.-]+)/n ) do
      '%' + $1.unpack( 'H2' * $1.size ).join( '%' ).upcase
    end.tr( ' ', '+' )
  end


  # Convert a string from CamelCase to ruby_case.
  #
  def Amazon.uncamelise(str)
    str.gsub( /(.[a-z])(?=[A-Z])/, "\\1_\\2" ).downcase
  end


  # The base class for items returned by Amazon.
  #
  class AmazonObject

    # Convert to a hash.
    #
    def to_h
      hash = {}
      self.properties.each do |property|
	key = property[1..-1]
	val = self.instance_variable_get( property )
	hash[key] = val
      end
      hash
    end


    # This alias makes the ability to determine an object's properties a
    # little more intuitive.
    #
    alias_method :properties, :instance_variables

    def initialize(url)	# :nodoc:
      @url = url
    end


    # Displays an object in a human-readable format. Call without a parameter,
    # as the parameter is only for internal use.
    # 
    def to_s(first_detail='productname')
      string = ""

      # Get all attributes, minus the leading '@'.
      vars = self.instance_variables.sort.map { |v| v[1..-1] }

      # Find the longest attribute name.
      longest = vars.sort { |a,b| b.length <=> a.length }[0].length

      # Put the product name at the front of the list, if we have one.
      vars.unshift( first_detail ) if vars.delete( first_detail )

      # Display the product's details.
      vars.each do |iv|
	value = self.instance_variable_get( "@#{iv}" ).inspect
	string << "%-#{longest}s = %s\n" % [ iv, value ]
      end

      string

    end


    # Fake the appearance of a product as a hash. _key_ should be any
    # attribute of the object and can be a String, Symbol or anything else
    # that can be made into a String with to_s.
    #
    def [](key)
      self.instance_variable_get( '@' + key.to_s )
    end


    def method_missing(method)
      iv = '@' + method.id2name

      if instance_variables.include? iv
        Amazon.module_eval { attr_reader method }
        instance_variable_get iv
      else
        nil
      end
    end
    private :method_missing

  end


  # A Class for dealing with configuration files, such as +/etc/amazonrc+
  # and +~/.amazonrc+.
  #
  class Config < Hash

    def initialize

      config_files = [ '/etc/amazonrc' ]
      config_files << File.expand_path( '~/.amazonrc' ) if ENV.key?( 'HOME' )
    
      config_files.each do |f|
        if File.exists?( f ) && File.readable?( f )

          Amazon.dprintf( 'Opening %s ...', f )
    
          File.open( f ) { |f| lines = f.readlines }.each do |line|
            line.chomp!
    
            # Skip comments and blank lines.
            #
            next if line =~ /^(#|$)/
    
            Amazon.dprintf( 'Read: %s', line )
    
            # Store these, because we'll probably find a use for these later.
            #
            begin
      	    match = line.match( /^(\S+)\s*=\s*(['"]?)([^'"]+)(['"]?)/ )
              key, begin_quote, val, end_quote = match[1, 4]
              raise ConfigError if begin_quote != end_quote
            rescue NoMethodError, ConfigError
              raise ConfigError, "bad config line: #{line}"
            end
    
            self[key] = val
    
          end
        end

      end

    end
  end


end
