# $Id: search.rb,v 1.4 2008/03/24 16:18:43 ianmacd Exp $
#

module Amazon

  module AWS

    require 'net/http'
    require 'rexml/document'

    module Search

      # Find items that are sold on www.amazon.com by merchants and most
      # sellers.
      #
      def item_search
	# Can use BrowseNodes response group.
      end

      # Find items that are only in a seller's store.
      #
      def seller_listing_search
      end

      # Find customer-created lists in which customers specify items they
      # would like to receive as presents.
      #
      def list_search
      end

      # Find items that are similar to the ones you've already found.
      #
      def similarity_lookup
	# Can use BrowseNodes response group.
      end

      # Returns desciptions of specified items.
      #
      def item_lookup
	# Can use BrowseNodes response group.
      end

      # Returns descriptions of items on a customer's list.
      #
      def list_lookup
	# Can use BrowseNodes response group.
      end

      # Returns descriptions of specified items in a seller's store.
      #
      def seller_listing_lookup
      end

      # Search for customers.
      #
      def customer_content_search
      end

      # Look up public information about a specific customer, such as their
      # wishlists.
      #
      def customer_content_lookup
      end

      # Returns information about a specific seller.
      #
      def seller_lookup
      end

      # Amazon uses browse nodes as a way of organising the millions of items
      # in their inventory.
      #
      def browse_node_lookup
	# Can use BrowseNodeInfo response group.
      end

      # Returns information about AWS operations and response groups.
      #
      def help
      end

      # Returns information about a customer's purchase.
      #
      def transaction_lookup
      end

      class Request

	# Exception class for bad access key ID.
	#
	class AccessKeyIdError < StandardError; end

	# Exception class for bad locales.
	#
	class LocaleError < StandardError; end

	attr_reader :cache, :conn, :locale, :user_agent

	def initialize(key_id=nil, associate=nil, locale=nil, cache=nil,
		       user_agent = USER_AGENT)

	  @config ||= Amazon::Config.new

	  def_locale = locale
	  locale = 'us' unless locale
	  locale.downcase!

	  key_id ||= @config['key_id']
	  cache ||= @config['cache']

	  # Take locale from config file if no locale was passed to method.
	  #
	  if @config.key?( 'locale' ) && ! def_locale
	    locale = @config['locale']
	  end
	  validate_locale( locale )

	  if key_id.nil?
	    raise AccessKeyIdError, 'key_id may not be nil'
	  end

	  @key_id     = key_id
	  @tag	      = associate || @config['associate'] || DEF_ASSOC[locale]
	  @user_agent = user_agent
	  @cache      = unless cache == 'false'
			  Amazon::AWS::Cache.new( @config['cache_dir'] ||
	  					  '/tmp/amazon' )
	  	        else
	  		  nil
	  	        end
	  self.locale = locale
        end

	def locale=(l)
	  old_locale = @locale ||= nil
	  @locale = validate_locale( l )

	  # Use the new locale's default ID if the ID currently in use is the
	  # current locale's default ID.
	  #
	  if @tag == Amazon::AWS::DEF_ASSOC[old_locale]
	    @tag = Amazon::AWS::DEF_ASSOC[@locale]
	  end

	  # We must now set up a new HTTP connection to the correct server for
	  # this locale, unless the same server is used for both.
	  #
	  unless Amazon::AWS::ENDPOINT[@locale] ==
		 Amazon::AWS::ENDPOINT[old_locale]
	    #connect( @locale )
	    @conn = nil
	  end
	end


	# Verify the validity of a locale string.
	#
	def validate_locale(l)
	  unless Amazon::AWS::ENDPOINT.has_key? l
	    raise LocaleError, "invalid locale: #{l}"
	  end
	  l
	end
	private :validate_locale

	# Return an HTTP connection for the current locale.
	#
	def connect(locale)
	  if ENV.key? 'http_proxy'
	    uri = URI.parse( ENV['http_proxy'] )
	    proxy_user = proxy_pass = nil
	    proxy_user, proxy_pass = uri.userinfo.split( /:/ ) if uri.userinfo
	    @conn = Net::HTTP::Proxy( uri.host, uri.port, proxy_user,
				      proxy_pass).start(
				    Amazon::AWS::ENDPOINT[locale] )
	  else
	    @conn = Net::HTTP::start( Amazon::AWS::ENDPOINT[locale].host )
	  end
	end
	private :connect

	# Reconnect to the server if our connection has been lost (due to a
	# time-out, etc.).
	#
	def reconnect
	  connect( self.locale )
	  self
	end


	def search(operation, response_group, nr_pages=1)
	  q_params = Amazon::AWS::SERVICE.
		       merge( { 'AWSAccessKeyId' => @key_id,
				'AssociateTag'	 => @tag } ).
		       merge( operation.params ).
		       merge( response_group.params )

	  query = Amazon::AWS::assemble_query( q_params )
	  page = Amazon::AWS::get_page( self, query )
	  r = Response.new( page, operation )

	  # If only one page has been requested or only one page is available,
	  # we can stop here.
	  #
	  if nr_pages == 1 || ( tp = r.total_pages ) == '1'
	     yield r if block_given?
	     return r
	  end

	  # Limit the number of pages to the maximum number available.
	  #
	  nr_pages = tp.to_i if nr_pages == :ALL_PAGES || nr_pages > tp.to_i

	  # Iterate over pages 2 and higher, but go no higher than MAX_PAGES.
	  #
	  2.upto( nr_pages < MAX_PAGES ? nr_pages : MAX_PAGES ) do |page_nr|
	    query = Amazon::AWS::assemble_query(
		      q_params.merge( { 'ItemPage' => page_nr } ) )
	    page = Amazon::AWS::get_page( self, query )

	    # Merge this page's results with the first page.
	    #
	    r.merge( Response.new( page, operation ) )
	  end

	  yield r if block_given?
	  r
	end

      end

      class Response
	include REXML

	attr_reader :operation_request, :item_sets, :seller_listing_sets,
		    :browse_node_sets, :list_set, :total_pages

	def initialize(page, operation)
	  doc = Document.new( page )

	  @operation_request = Amazon::AWS::OperationRequest.new( doc )

	  case operation
	  when Amazon::AWS::ItemSearch, Amazon::AWS::ItemLookup
	    @item_sets = Amazon::AWS::ItemSet.new( doc )
	    @total_pages = @item_sets.item_requests[0].total_pages
	  when Amazon::AWS::SellerListingSearch
	    @seller_listing_sets = Amazon::AWS::SellerListingSet.new( doc )
	    @total_pages = @seller_listing_sets.seller_listing_requests[0].total_pages
	  when Amazon::AWS::BrowseNodeLookup
	    @browse_node_sets = Amazon::AWS::BrowseNodeLookupSet.new( doc )
	    @total_pages = @browse_node_sets.browse_node_requests[0].total_pages
	  when Amazon::AWS::ListSearch
	    @list_set = Amazon::AWS::ListSet.new( doc )
	    @total_pages = @list_set.list_search_request.total_pages
	  else
	    # FIXME: shouldn't reach here
	    raise
	  end
	end

	def merge(responses)
	  responses = [ responses ] unless responses.kind_of? Array
	  responses.each do |r|
	    case r.operation_request.arguments['Operation']
	    when 'ItemSearch'
	      @item_sets.items.concat( r.item_sets.items )
	    when 'ListSearch'
	      @list_set.lists.concat( r.list_set.lists )
	    else
	      # FIXME: shouldn't reach here
	      raise
	    end

	  end
	end

      end

    end
  end
end
